/*
 * Xerces.java
 *
 * Created on February 18, 2004, 3:19 PM
 */

package com.milowski.jaxp;

import java.io.*;
import org.apache.xerces.util.XMLCatalogResolver;
import org.apache.xml.resolver.CatalogManager;
import org.apache.xml.resolver.helpers.Debug;
import javax.xml.parsers.*;
import org.xml.sax.*;

/**
 *
 *
 * @author  alex
 */
public class XParse {
   
   private static Debug debug = CatalogManager.getStaticManager().debug;

   static class DisplayErrors implements ErrorHandler {
      
      public void error(org.xml.sax.SAXParseException sAXParseException) throws org.xml.sax.SAXException {
         System.err.println(sAXParseException.getMessage());
      }
      
      public void fatalError(org.xml.sax.SAXParseException sAXParseException) throws org.xml.sax.SAXException {
         System.err.println(sAXParseException.getMessage());
      }
      
      public void warning(org.xml.sax.SAXParseException sAXParseException) throws org.xml.sax.SAXException {
         System.err.println(sAXParseException.getMessage());
      }
      
   }
   /** Creates a new instance of XParse */
   public XParse() {
   }
   
   public static void usage() {
      System.err.println("XParse { -c catalog | -catalog catalog | -v | -validate } uri");
   }
   
   public static void main(String [] args) {
      int argidx = 0;
      String catalogFile = null;
      boolean validate = false;
      while (argidx<args.length && args[argidx].charAt(0)=='-') {
         if (args[argidx].equals("-c") || args[argidx].equals("-catalog")) {
            argidx++;
            if (argidx==args.length) {
               System.err.println(args[argidx-1]+" option requires a parameter of a catalog file.");
               System.exit(1);
            }
            catalogFile = args[argidx];
         } else if (args[argidx].equals("-v") || args[argidx].equals("-validate")) {
            validate = true;
         } else {
            System.err.println("Unknown option: "+args[argidx]);
            System.exit(1);
         }
         argidx++;
      }
      if ((args.length-argidx)!=1) {
         usage();
         System.exit(1);
      }
      try {
         SAXParserFactory pfactory = SAXParserFactory.newInstance();
         pfactory.setNamespaceAware(true);
         pfactory.setValidating(true);
         SAXParser parser = pfactory.newSAXParser();
         if (validate) {
            parser.setProperty(
               "http://java.sun.com/xml/jaxp/properties/schemaLanguage",
               "http://www.w3.org/2001/XMLSchema"
            );
         }
         XMLReader xmlReader = parser.getXMLReader();
         
         xmlReader.setErrorHandler(new DisplayErrors());
         
         if (catalogFile!=null) {
            String catalogURI = (new File(catalogFile)).toURI().toString();
            //System.out.println("Setting catalog to "+catalogURI);
            String [] catalogs = { catalogURI };

            // Create catalog resolver and set a catalog list.
            XMLCatalogResolver resolver = new XMLCatalogResolver();
            resolver.setPreferPublic(true);
            resolver.setCatalogList(catalogs);

            // Set the resolver on the parser.
            xmlReader.setProperty(
              "http://apache.org/xml/properties/internal/entity-resolver", 
              resolver);            
            
         }
         
         xmlReader.parse(args[argidx]);
      } catch (ParserConfigurationException ex) {
         ex.printStackTrace();
      } catch (org.xml.sax.SAXException ex) {
         ex.printStackTrace();
      } catch (java.io.FileNotFoundException ex) {
         System.err.println("Cannot open "+ex.getMessage());
      } catch (java.io.IOException ex) {
         ex.printStackTrace();
      }
   }
   
}
