/*
 * XSLTFilter.java
 *
 * Created on January 16, 2005, 11:07 AM
 */

package com.milowski.jaxp;


import java.io.*;
import java.net.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import javax.xml.transform.*;
import javax.xml.transform.stream.*;
import javax.xml.transform.dom.*;

/**
 *
 * @author R. Alexander Milowski
 */
public class XSLTFilter implements Filter
{
   static class ServletByteArrayOutputStream extends ServletOutputStream {
      
      ByteArrayOutputStream os;

      ServletByteArrayOutputStream() {
         os = new ByteArrayOutputStream();
      }
      public void write(int param) throws java.io.IOException
      {
         os.write(param);
      }
      
      public void write(byte [] b) throws java.io.IOException
      {
         os.write(b);
      }
      
      public void write(byte [] b,int off,int len) throws java.io.IOException
      {
         os.write(b,off,len);
      }
      
      public String toString() {
         return os.toString();
      }
      
      public int size() {
         return os.size();
      }
      
   }

   static class StringResponseWrapper extends HttpServletResponseWrapper {
      CharArrayWriter out;
      ServletByteArrayOutputStream bos;
      ServletContext context;
      int code;
      
      StringResponseWrapper(HttpServletResponse response) {
         super(response);
         out = null;
         bos = null;
         context = null;
      }
      
      public void setContext(ServletContext context) {
         this.context = context;
      }
      
      public int getStatusCode() {
         return code;
      }
      
      public PrintWriter getWriter()
         throws IOException
      {
         
         if (context!=null) {
            context.log("Called getWriter():");
         }
         /*
         if (out==null && bos==null) {
            out = new CharArrayWriter();
         }*/
         if (out==null) {
            out = new CharArrayWriter();
         }
         return new PrintWriter(out);
      }
      
      public ServletOutputStream getOutputStream()
         throws IOException
      {
         if (context!=null) {
            context.log("Called getOutputStream():");
         }
         if (bos==null) {
            bos = new ServletByteArrayOutputStream();
         }
         return bos;
      }
      
      public String toString() {
         try {
            if (out!=null) {
               out.flush();
            }
            if (bos!=null) {
               bos.flush();
            }
         } catch (IOException ex) {
            
         }
         if (context!=null && out!=null) {
            context.log("Char array size="+out.size());
         }
         if (out==null && bos==null) {
            return "";
         }
         if (out==null || bos!=null && out.size()==0) {
            return bos.toString();
         } else {
            return out.toString();
         }
      }
      
      public void setContentLength(int len) {
         
      }
      
      public void setStatus(int code) {
         super.setStatus(code);
         this.code = code;
      }
      
   }
   
   static class TransformationInfo {
      long timestamp;
      Transformer xform;
      TransformationInfo(Transformer xform) {
         this.timestamp = System.currentTimeMillis();
         this.xform = xform;
      }
   }
   
   
   boolean reload;
   Map cache;
   TransformerFactory tfactory;
   Transformer transformation;
   File transformationFile;
   long timestamp;
   URL stylesheetLocation;
   ServletContext context;
   String contentType;
   Map staticExtensions;
   Map extensions;
   
   /** Creates a new instance of XSLTFilter */
   public XSLTFilter()
   {
   }
   
   public void destroy()
   {
   }
   
   public void doFilter(ServletRequest servletRequest, ServletResponse servletResponse, FilterChain filterChain) 
      throws java.io.IOException, ServletException
   {
      if (transformation==null && stylesheetLocation!=null) {
         try {
            transformation = getStylesheet(stylesheetLocation);
         } catch (TransformerConfigurationException ex) {
            context.log("Cannot load stylesheet for request "+stylesheetLocation,ex);
            throw new ServletException("Cannot load static stylesheet "+stylesheetLocation+" for resource.");
         }
      }
      HttpServletRequest hrequest = (HttpServletRequest)servletRequest;
      HttpServletResponse hresponse = (HttpServletResponse)servletResponse;
      String path = hrequest.getServletPath();
      String origPath = path;
      File inputFile = new File(context.getRealPath(origPath));
      URI baseURI = inputFile.toURI();
      String extension = null;
      int dot = path.lastIndexOf('.');
      if (dot<0) {
         path = path+".xsl";
      } else {
         extension = path.substring(dot+1);
         path = path.substring(0,dot)+".xsl";
      }
      context.log("Extension='"+extension+"'");
      if (extensions!=null && extensions.get(extension)==null) {
         // Skip this extension
         context.log("Skipping.");
         filterChain.doFilter(servletRequest, servletResponse);
         return;
      }
      if (transformation==null) {
         context.log("Finding specific stylesheet...");
         //context.log("Request URI: '"+path+"'");
         //context.log("Path Translated: '"+hrequest.getPathTranslated()+"'");
         //context.log("Context Path: '"+hrequest.getContextPath()+"'");
         //context.log("Path to stylesheet: '"+path+"'");
         String fpath = context.getRealPath(path);
         //context.log("Real path to stylesheet: '"+fpath+"'");
         File f = new File(fpath);
         if (f.exists()) {
            URL location = f.toURL();
            //context.log("Exists!  Loading from '"+location+"'");
            try {

               Transformer xform = getStylesheet(location);
               
               Source source = null;
               if (staticExtensions!=null && staticExtensions.get(extension)!=null) {
                  context.log("Processing file statically.");
                  source = new StreamSource(inputFile);
               } else {
                  StringResponseWrapper wrapper = new StringResponseWrapper(((HttpServletResponse)servletResponse));
                  filterChain.doFilter(servletRequest, wrapper);

                  String docString = wrapper.toString();
                  if (docString.length()!=0) {
                     source = new StreamSource(new StringReader(docString));
                  }
                  
               }

               servletResponse.setContentType(contentType);
               if (source!=null) {
                  hresponse.setHeader("Pragma", "No-cache");
                  hresponse.setDateHeader("Expires", 0);
                  hresponse.setHeader("Cache-Control", "no-cache");
                  hresponse.setCharacterEncoding("UTF-8");
                  try {
                     PrintWriter out = servletResponse.getWriter();
                     xform.transform(source,new StreamResult(out));
                     out.flush();
                  } catch (TransformerException ex) {
                     context.log("Cannot apply stylesheet for request "+location,ex);
                     throw new ServletException("Cannot load static stylesheet "+location+" for request.");
                  }
               }
               
            } catch (TransformerConfigurationException ex) {
               context.log("Cannot load stylesheet for request "+location,ex);
               throw new ServletException("Cannot load static stylesheet "+location+" for request.");
            }
         } else {
            filterChain.doFilter(servletRequest, servletResponse);
         }
      } else {
         if (transformationFile.lastModified()>timestamp) {
            try {
               transformation = getStylesheet(stylesheetLocation);
            } catch (TransformerConfigurationException ex) {
               context.log("Cannot load stylesheet "+stylesheetLocation+" for request",ex);
               throw new ServletException("Cannot load stylesheet "+stylesheetLocation+" for request.");
            }
            timestamp = transformationFile.lastModified();
         }

         StreamSource source = null;
         if (staticExtensions!=null && staticExtensions.get(extension)!=null) {
            context.log("Processing file statically.");
            source = new StreamSource(inputFile);
         } else {
            context.log("Calling chain.");
            StringResponseWrapper wrapper = new StringResponseWrapper(((HttpServletResponse)servletResponse));
            filterChain.doFilter(servletRequest, wrapper);

            String docString = wrapper.toString();
            if (docString.length()!=0) {
               source = new StreamSource(new StringReader(docString));
            }
         }

         servletResponse.setContentType(contentType);
         if (source!=null) {
            hresponse.setHeader("Pragma", "No-cache");
            hresponse.setDateHeader("Expires", 0);
            hresponse.setHeader("Cache-Control", "no-cache");
            hresponse.setCharacterEncoding("UTF-8");
            try {
               transformation.transform(source, new StreamResult(servletResponse.getWriter()));
            } catch (TransformerException ex) {
               context.log("Cannot apply stylesheet for request.",ex);
               throw new ServletException("Cannot apply stylesheet for resource.");
            }
         }
      }
   }
   
   public void init(FilterConfig filterConfig) 
      throws ServletException
   {
      context = filterConfig.getServletContext();
      tfactory = TransformerFactory.newInstance();
      contentType = filterConfig.getInitParameter("content-type");
      if (contentType==null) {
         contentType = "text/html";
      }
      extensions = null;
      String extensionsSpec = filterConfig.getInitParameter("extensions");
      if (extensionsSpec!=null) {
         String [] toAdd = extensionsSpec.split(",");
         extensions = new HashMap();
         for (int i=0; i<toAdd.length; i++) {
            extensions.put(toAdd[i],Boolean.TRUE);
         }
      }
      try {
         String relPath = filterConfig.getInitParameter("stylesheet");
         if (relPath!=null) {
            String path = context.getRealPath(relPath);
            transformationFile = new File(path);
            timestamp = transformationFile.lastModified();
            cache = null;
            stylesheetLocation = (new File(path)).toURL();
            transformation = getStylesheet(stylesheetLocation);
         } else {
            cache = new HashMap();
            transformation = null;
         }
         String spec = filterConfig.getInitParameter("reload");
         reload = spec!=null && spec.equals("yes") ? true : false;
      } catch (java.io.IOException ex) {
         throw new ServletException("Cannot load stylesheet due to I/O error: "+ex.getMessage(),ex);
      } catch (TransformerConfigurationException ex) {
         throw new ServletException("Error in stylesheet: "+ex.getMessage(),ex);
      }
      String extConfig = filterConfig.getInitParameter("static-extensions");
      if (extConfig!=null) {
         staticExtensions = new HashMap();
         String [] exts = extConfig.split("\\s");
         for (int i=0; i<exts.length; i++) {
            context.log(filterConfig.getFilterName()+": Adding static extension "+exts[i]);
            staticExtensions.put(exts[i],Boolean.TRUE);
         }
      } else {
         staticExtensions = null;
      }
   }
   
   public Transformer getStylesheet(URL location) 
      throws TransformerConfigurationException
   {

      if (cache!=null) {
         TransformationInfo tinfo = (TransformationInfo)cache.get(location);
         if (tinfo!=null && reload) {
            File f = new File(location.getPath());
            if (!f.exists()) {
               tinfo = null;
            } else if (f.lastModified()>tinfo.timestamp) {
               tinfo = null;
            }
         }
         if (tinfo!=null) {
            return tinfo.xform;
         }
      }

      context.log("Loading stylesheet from location "+location);

      Transformer xform = tfactory.newTransformer(new StreamSource(location.toString()));

      if (cache!=null) {
         cache.put(location,new TransformationInfo(xform));
      }
      return xform;
      
   }
}
